/****************************************************************************************** 
  Fichier exemple issu du tutoriel : 
  "Compilation et dploiement d'un modle Simulink avec Embedded Coder"
   
  http://simulink.developpez.com/tutoriels/compilation-deploiement-modele-embedded-coder
   
  Auteurs : duf42 et Jrme Briot pour Developpez.com - 2013   	
 ******************************************************************************************/

#if (defined(_WIN32)||defined(_WIN64)) /* Windows */
#include <windows.h>
#define GETSYMBOLADDR GetProcAddress
#define LOADLIB LoadLibrary
#define CLOSELIB FreeLibrary
 
#else /* UNIX - Linux - Mac */
#include <dlfcn.h>
#define GETSYMBOLADDR dlsym
#define LOADLIB dlopen
#define CLOSELIB dlclose
 
#endif
 
#if defined(_WIN64) /* Windows 64 bits */
#define LIBNAME "./gestion_reservoir_win64.dll"
#elif defined(_WIN32) /* Windows 32 bits */
#define LIBNAME "./gestion_reservoir_win32.dll"
#else /* UNIX - Linux - Mac */
#define LIBNAME "./gestion_reservoir.so"
#endif
 
#include "gestion_reservoir/gestion_reservoir.h"
#include <stdio.h>
#include <stdlib.h>
 
#define N 600
 
int main(void) {
 
    boolean_T firstTime = 1;
    int i;
 
    void *handleLib;
    void (*mdl_initialize)(boolean_T);
    void (*mdl_step)(void);
    void (*mdl_terminate)(void);
 
    double niveau_courant[N];
    double debit_courant[N];
 
    double niveau_cible = 2.0;
    double debit_cible = 0.0003;
 
    ExternalInputs_gestion_reserv_T (*mdl_Uptr);
    ExternalOutputs_gestion_reser_T (*mdl_Yptr);
 
    #if (defined(_WIN32)||defined(_WIN64)) /* Windows */
    handleLib = LOADLIB(LIBNAME);
    #else /* UNIX - Linux - Mac */
    handleLib = LOADLIB(LIBNAME, RTLD_LAZY);
    #endif

    mdl_initialize = (void(*)(boolean_T))GETSYMBOLADDR(handleLib , "gestion_reservoir_initialize");
    mdl_terminate = (void(*)(void))GETSYMBOLADDR(handleLib , "gestion_reservoir_terminate");
    mdl_step = (void(*)(void))GETSYMBOLADDR(handleLib , "gestion_reservoir_step");
 
    mdl_Uptr = (ExternalInputs_gestion_reserv_T*)GETSYMBOLADDR(handleLib , "gestion_reservoir_U");
    mdl_Yptr = (ExternalOutputs_gestion_reser_T*)GETSYMBOLADDR(handleLib , "gestion_reservoir_Y");
 
    mdl_initialize(firstTime);	
	
    for(i=0;i<N;i++) {
 
        mdl_Uptr->niveau_cible = niveau_cible;
        mdl_Uptr->debit_cible = debit_cible;
 
        mdl_step();
 
        niveau_courant[i] = mdl_Yptr->niveau_courant;
        debit_courant[i] = mdl_Yptr->debit_courant;
 
    }
 
    mdl_terminate();
 
    for(i=0;i<N;i++)
        printf("%.6f %.6f\n",niveau_courant[i],debit_courant[i]);

 
    CLOSELIB(handleLib);
 
    return 0;
 
}